from .death_report_extra_fields_model_mixin import DeathReportExtraFieldsModelMixin
from .death_report_model_mixin import DeathReportModelMixin
from .death_report_tmg_model_mixin import (
    DeathReportTmgModelMixin,
    DeathReportTmgSecondManager,
    DeathReportTmgSecondModelMixin,
    DeathReportTmgSecondSiteManager,
)
from .simple_death_report_model_mixin import SimpleDeathReportModelMixin



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_constants.constants import CLOSED, NO
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin


class DefaultAeTmgFormValidator(FormValidator):
    def clean(self):

        self.validate_other_specify(field="ae_classification")

        self.required_if(NO, field="original_report_agreed", field_required="narrative")

        self.required_if(
            CLOSED, field="report_status", field_required="report_closed_datetime"
        )


class AeTmgModelFormMixin(
    FormValidatorMixin, ModelFormSubjectIdentifierMixin, ActionItemFormMixin
):

    form_validator_cls = DefaultAeTmgFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
        help_text="(read-only)",
    )

    ae_description = forms.CharField(
        label="Original AE Description",
        required=False,
        widget=forms.Textarea(attrs={"readonly": "readonly", "cols": "79"}),
        help_text="(read-only)",
    )

    ae_classification = forms.CharField(
        label="AE Classification",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
        help_text="(read-only)",
    )

    ae_classification_other = forms.CharField(
        label="AE Classification (if `other` above)",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
        help_text="(read-only)",
    )



from django.conf import settings
from django.db import models
from django.db.models.deletion import PROTECT
from edc_action_item.managers import (
    ActionIdentifierManager,
    ActionIdentifierSiteManager,
)
from edc_action_item.models import ActionModelMixin
from edc_constants.choices import YES_NO
from edc_identifier.model_mixins import (
    NonUniqueSubjectIdentifierFieldMixin,
    TrackingModelMixin,
)
from edc_model.models import ReportStatusModelMixin, datetime_not_future
from edc_protocol.validators import datetime_not_before_study_start
from edc_sites.models import SiteModelMixin
from edc_utils import get_utcnow

from edc_adverse_event.models import CauseOfDeath

from ...constants import DEATH_REPORT_TMG_ACTION, DEATH_REPORT_TMG_SECOND_ACTION


class DeathReportTmgManager(ActionIdentifierManager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(action_item__action_type__name=DEATH_REPORT_TMG_ACTION)


class DeathReportTmgSiteManager(ActionIdentifierSiteManager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(action_item__action_type__name=DEATH_REPORT_TMG_ACTION)


class DeathReportTmgSecondManager(ActionIdentifierManager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(action_item__action_type__name=DEATH_REPORT_TMG_SECOND_ACTION)


class DeathReportTmgSecondSiteManager(ActionIdentifierSiteManager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(action_item__action_type__name=DEATH_REPORT_TMG_SECOND_ACTION)


class DeathReportTmgFieldsModelMixin(models.Model):

    death_report = models.ForeignKey(
        f"{settings.ADVERSE_EVENT_APP_LABEL}.deathreport", on_delete=PROTECT
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report Date",
        validators=[datetime_not_before_study_start, datetime_not_future],
        default=get_utcnow,
    )

    cause_of_death = models.ForeignKey(
        CauseOfDeath,
        on_delete=PROTECT,
        verbose_name=("Main cause of death"),
        help_text=(
            "Main cause of death in the opinion of the " "local study doctor and local PI"
        ),
        null=True,
    )

    cause_of_death_other = models.CharField(
        verbose_name='If "Other" above, please specify',
        max_length=100,
        blank=True,
        null=True,
    )

    cause_of_death_agreed = models.CharField(
        verbose_name=("Is the cause of death agreed between study doctor and TMG member?"),
        max_length=15,
        choices=YES_NO,
        blank=True,
        null=True,
        help_text="If No, explain in the narrative below",
    )

    narrative = models.TextField(verbose_name="Narrative", blank=True, null=True)

    class Meta:
        abstract = True


class DeathReportTmgMethodsModelMixin(models.Model):
    def __str__(self):
        return str(self.death_report)

    def save(self, *args, **kwargs):
        self.subject_identifier = self.death_report.subject_identifier
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.action_identifier,)

    natural_key.dependencies = ["edc_adverse_event.causeofdeath"]

    class Meta:
        abstract = True


class DeathReportTmgModelMixin(
    NonUniqueSubjectIdentifierFieldMixin,
    ActionModelMixin,
    TrackingModelMixin,
    DeathReportTmgFieldsModelMixin,
    DeathReportTmgMethodsModelMixin,
    ReportStatusModelMixin,
    SiteModelMixin,
    models.Model,
):

    action_name = DEATH_REPORT_TMG_ACTION

    tracking_identifier_prefix = "DT"

    on_site = DeathReportTmgSiteManager()

    objects = DeathReportTmgManager()

    class Meta:
        abstract = True
        verbose_name = "Death Report TMG (1st)"
        verbose_name_plural = "Death Report TMG (1st)"
        indexes = [
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])
        ]


class DeathReportTmgSecondModelMixin(DeathReportTmgModelMixin):

    action_name = DEATH_REPORT_TMG_SECOND_ACTION

    tracking_identifier_prefix = "DT"

    on_site = DeathReportTmgSecondSiteManager()

    objects = DeathReportTmgSecondManager()

    class Meta:
        abstract = True
        verbose_name = "Death Report TMG (2nd)"
        verbose_name_plural = "Death Report TMG (2nd)"
        indexes = [
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])
        ]



from django import forms
from edc_action_item.modelform_mixins import ActionItemModelFormMixin

from ..get_ae_model import get_ae_model
from .mixins import AeFollowupModelFormMixin


class AeFollowupForm(AeFollowupModelFormMixin, ActionItemModelFormMixin, forms.ModelForm):
    class Meta:
        model = get_ae_model("aefollowup")
        fields = "__all__"



from django import forms
from django.conf import settings
from django.urls.base import reverse
from django.utils.safestring import mark_safe
from edc_action_item.forms import ActionItemFormMixin
from edc_constants.constants import YES
from edc_form_validators import FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin
from edc_reportable import GRADE4, GRADE5

from edc_adverse_event.form_validators import AeInitialFormValidator
from edc_adverse_event.get_ae_model import get_ae_model


class AeInitialModelFormMixin(
    FormValidatorMixin, ModelFormSubjectIdentifierMixin, ActionItemFormMixin
):

    form_validator_cls = AeInitialFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        cleaned_data = super().clean()
        self.raise_if_followup_exists()
        self.validate_sae_and_grade()
        return cleaned_data

    def validate_sae_and_grade(self):
        """Raise an exception if grade>=4 and user did not
        indicate that this is an SAE.
        """
        if (
            self.cleaned_data.get("ae_grade") in [GRADE4, GRADE5]
            and self.cleaned_data.get("sae") != YES
        ):
            raise forms.ValidationError({"sae": "Invalid. Grade is >= 4"})

    @property
    def changelist_url(self):
        AeFollowup = get_ae_model("aefollowup")
        app_label = AeFollowup._meta.app_label
        model_name = AeFollowup._meta.object_name.lower()
        return reverse(
            f"{settings.ADVERSE_EVENT_ADMIN_SITE}:{app_label}_{model_name}_changelist"
        )

    def raise_if_followup_exists(self):
        """Raise an exception if the AE followup exists
        and the user is attempting to change this form.
        """
        AeFollowup = get_ae_model("aefollowup")
        if AeFollowup.objects.filter(ae_initial=self.instance.pk).exists():
            url = f"{self.changelist_url}?q={self.instance.action_identifier}"
            raise forms.ValidationError(
                mark_safe(
                    f"Unable to save. Follow-up reports exist. Provide updates "
                    f"to this report using the "
                    f"{AeFollowup._meta.verbose_name} instead. "
                    f'See <A href="{url}">AE Follow-ups for {self.instance}</A>.'
                )
            )



from django import forms
from edc_action_item.modelform_mixins import ActionItemModelFormMixin

from ..get_ae_model import get_ae_model
from .mixins import DeathReportTmgModelFormMixin


class DeathReportTmgSecondForm(
    DeathReportTmgModelFormMixin, ActionItemModelFormMixin, forms.ModelForm
):
    class Meta:
        model = get_ae_model("deathreporttmgsecond")
        fields = "__all__"



from django.conf import settings
from edc_model_wrapper import ModelWrapper


class AeInitialModelWrapper(ModelWrapper):

    model = f"{settings.ADVERSE_EVENT_APP_LABEL}.aeinitial"
    next_url_name = "ae_listboard_url"
    next_url_attrs = ["subject_identifier"]

    @property
    def pk(self):
        return str(self.object.pk)

    @property
    def subject_identifier(self):
        return self.object.subject_identifier



from .ae_initial_model_wrapper import AeInitialModelWrapper  # noqa
from .ae_tmg_model_wrapper import AeTmgModelWrapper
from .death_report_model_wrapper import DeathReportModelWrapper  # noqa
from .death_report_tmg_model_wrapper import DeathReportTmgModelWrapper  # noqa
from .death_report_tmg_second_model_wrapper import (  # noqa
    DeathReportTmgSecondModelWrapper,
)
from .tmg import (
    ClosedTmgActionItemModelWrapper,
    NewTmgActionItemModelWrapper,
    OpenTmgActionItemModelWrapper,
    TmgActionItemModelWrapper,
)



from django.conf import settings
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls.base import reverse
from django.utils.safestring import mark_safe
from edc_action_item import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ModelAdminActionItemMixin
from edc_constants.constants import DEAD
from edc_model_admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from edc_adverse_event.get_ae_model import get_ae_model

from ..forms import AeInitialForm
from ..templatetags.edc_adverse_event_extras import (
    format_ae_description,
    select_description_template,
)
from .modeladmin_mixins import AdverseEventModelAdminMixin

fieldset_part_one = (
    "Part 1: Description",
    {
        "fields": (
            "ae_classification",
            "ae_classification_other",
            "ae_description",
            "ae_awareness_date",
            "ae_start_date",
            "ae_grade",
        )
    },
)

fieldset_part_two = (
    "Part 2: Cause and relationship to study",
    {
        "fields": (
            "ae_study_relation_possibility",
            "study_drug_relation",
            "ae_cause",
            "ae_cause_other",
        )
    },
)

fieldset_part_three = ("Part 3: Treatment", {"fields": ("ae_treatment",)})

fieldset_part_four = (
    "Part 4: SAE / SUSAR",
    {"fields": ("sae", "sae_reason", "susar", "susar_reported")},
)

default_radio_fields = {
    "ae_cause": admin.VERTICAL,
    "ae_classification": admin.VERTICAL,
    "ae_grade": admin.VERTICAL,
    "ae_study_relation_possibility": admin.VERTICAL,
    "study_drug_relation": admin.VERTICAL,
    "sae": admin.VERTICAL,
    "sae_reason": admin.VERTICAL,
    "susar": admin.VERTICAL,
    "susar_reported": admin.VERTICAL,
}


class AeInitialModelAdminMixin(
    AdverseEventModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    ModelAdminActionItemMixin,
):

    form = AeInitialForm

    email_contact = settings.EMAIL_CONTACTS.get("ae_reports")
    additional_instructions = mark_safe(
        "Complete the initial AE report and forward to the TMG. "
        f'Email to <a href="mailto:{email_contact}">{email_contact}</a>'
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        fieldset_part_one,
        fieldset_part_two,
        fieldset_part_three,
        fieldset_part_four,
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = default_radio_fields

    ordering = ["-tracking_identifier"]

    list_display = [
        "identifier",
        "dashboard",
        "description",
        "follow_up_reports",
        "user",
    ]

    list_filter = [
        "ae_awareness_date",
        "ae_grade",
        "ae_classification",
        "sae",
        "sae_reason",
        "susar",
        "susar_reported",
    ]

    search_fields = ["subject_identifier", "action_identifier", "tracking_identifier"]

    def if_sae_reason(self, obj):
        """Returns the SAE reason.

        If DEATH, adds link to the death report.
        """
        if obj.sae_reason.name == DEAD:
            DeathReport = get_ae_model("deathreport")
            try:
                death_report = DeathReport.objects.get(
                    subject_identifier=obj.subject_identifier
                )
            except ObjectDoesNotExist:
                link = '<font color="red">Death report pending</font>'
            else:
                url_name = f"{settings.ADVERSE_EVENT_APP_LABEL}_deathreport"
                namespace = self.admin_site.name
                url = reverse(f"{namespace}:{url_name}_changelist")
                link = (
                    f'See report <a title="go to Death report"'
                    f'href="{url}?q={death_report.subject_identifier}">'
                    f"<span nowrap>{death_report.identifier}</span></a>"
                )
            return mark_safe(f"{obj.sae_reason.name}.<BR>{link}.")
        return obj.get_sae_reason_display()

    if_sae_reason.short_description = "If SAE, reason"

    def description(self, obj):
        """Returns a formatted comprehensive description of the SAE
        combining multiple fields.
        """
        context = format_ae_description({}, obj, 50)
        return render_to_string(select_description_template("aeinitial"), context)



from django.conf import settings
from edc_notification import GradedEventNotification, ModelNotification, register


@register()
class AeInitialG3EventNotification(GradedEventNotification):

    name = "g3_aeinitial"
    display_name = "Grade 3 initial event reported"
    grade = 3
    model = f"{settings.ADVERSE_EVENT_APP_LABEL}.aeinitial"


@register()
class AeFollowupG3EventNotification(GradedEventNotification):

    name = "g3_aefollowup"
    display_name = "Grade 3 followup event reported"
    grade = 3
    model = f"{settings.ADVERSE_EVENT_APP_LABEL}.aefollowup"


@register()
class AeInitialG4EventNotification(GradedEventNotification):

    name = "g4_aeinitial"
    display_name = "Grade 4 initial event reported"
    grade = 4
    model = f"{settings.ADVERSE_EVENT_APP_LABEL}.aeinitial"


@register()
class AeFollowupG4EventNotification(GradedEventNotification):

    name = "g4_aefollowup"
    display_name = "Grade 4 followup event reported"
    grade = 4
    model = f"{settings.ADVERSE_EVENT_APP_LABEL}.aefollowup"


@register()
class DeathNotification(ModelNotification):

    name = "death"
    display_name = "a death has been reported"
    model = f"{settings.ADVERSE_EVENT_APP_LABEL}.deathreport"



from django.conf import settings
from django.urls.base import reverse
from django.utils.safestring import mark_safe
from edc_utils.text import convert_php_dateformat

from ..get_ae_model import get_ae_model


class NonAeInitialModelAdminMixin:
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "ae_initial":
            if request.GET.get("ae_initial"):
                kwargs["queryset"] = get_ae_model("aeinitial").objects.filter(
                    id__exact=request.GET.get("ae_initial", 0)
                )
            else:
                kwargs["queryset"] = get_ae_model("aeinitial").objects.none()
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj=obj)
        if obj:
            fields = list(fields) + ["ae_initial"]
        return list(fields)

    def initial_ae(self, obj):
        """Returns a shortened action identifier."""
        if obj.ae_initial:
            url_name = "_".join(obj.ae_initial._meta.label_lower.split("."))
            namespace = self.admin_site.name
            url = reverse(f"{namespace}:{url_name}_changelist")
            return mark_safe(
                f'<a data-toggle="tooltip" title="go to ae initial report" '
                f'href="{url}?q={obj.ae_initial.action_identifier}">'
                f"{obj.ae_initial.identifier}</a>"
            )
        return None


class AdverseEventModelAdminMixin:
    def user(self, obj):
        """Returns formatted user names and creation/modification dates."""
        return mark_safe(
            "<BR>".join(
                [
                    obj.user_created,
                    obj.created.strftime(convert_php_dateformat(settings.SHORT_DATE_FORMAT)),
                    obj.user_modified,
                    obj.modified.strftime(convert_php_dateformat(settings.SHORT_DATE_FORMAT)),
                ]
            )
        )

    def follow_up_reports(self, ae_initial):
        """Returns a formatted list of links to AE Follow up reports."""
        followups = []
        AeFollowup = get_ae_model("aefollowup")
        AeSusar = get_ae_model("aesusar")
        for ae_followup in AeFollowup.objects.filter(
            related_action_item=ae_initial.action_item
        ):
            url = self.get_changelist_url(ae_followup)
            report_datetime = ae_followup.report_datetime.strftime(
                convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
            )
            followups.append(
                f'<a title="go to AE follow up report for '
                f'{report_datetime}" '
                f'href="{url}?q={ae_initial.action_identifier}">'
                f"<span nowrap>{ae_followup.identifier}</span></a>"
            )
        for ae_susar in AeSusar.objects.filter(related_action_item=ae_initial.action_item):
            url = self.get_changelist_url(ae_susar)
            report_datetime = ae_susar.report_datetime.strftime(
                convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
            )
            followups.append(
                f'<a title="go to AE SUSAR report for '
                f'{report_datetime}" '
                f'href="{url}?q={ae_initial.action_identifier}">'
                f"<span nowrap>{ae_susar.identifier} (SUSAR)</span></a>"
            )
        if followups:
            return mark_safe("<BR>".join(followups))
        return None

    def get_changelist_url(self, obj):
        url_name = "_".join(obj._meta.label_lower.split("."))
        namespace = self.admin_site.name
        return reverse(f"{namespace}:{url_name}_changelist")



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from edc_adverse_event.form_validators import DeathReportFormValidator


class DeathReportModelFormMixin(SiteModelFormMixin, ActionItemFormMixin, FormValidatorMixin):

    form_validator_cls = DeathReportFormValidator

    subject_identifier = forms.CharField(
        label="Subject identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )



from .ae_followup import AeFollowupModelFormMixin, validate_ae_initial_outcome_date
from .ae_initial import AeInitialModelFormMixin
from .ae_susar import AeSusarModelFormMixin
from .ae_tmg import AeTmgModelFormMixin
from .death_report import DeathReportModelFormMixin
from .death_report_tmg import DeathReportTmgModelFormMixin



import arrow
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from edc_dashboard.view_mixins import (
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
)
from edc_dashboard.views import ListboardView as BaseListboardView
from edc_navbar import NavbarViewMixin

from edc_adverse_event.constants import (
    AE_FOLLOWUP_ACTION,
    AE_TMG_ACTION,
    DEATH_REPORT_ACTION,
    DEATH_REPORT_TMG_ACTION,
)
from edc_adverse_event.model_wrappers import (
    TmgActionItemModelWrapper as BaseTmgActionItemModelWrapper,
)

from ...auth_objects import TMG


class TmgActionItemModelWrapper(BaseTmgActionItemModelWrapper):
    next_url_name = "tmg_summary_listboard_url"


class SummaryListboardView(
    NavbarViewMixin,
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    BaseListboardView,
):

    listboard_back_url = "tmg_home_url"

    ae_tmg_model = f"{settings.ADVERSE_EVENT_APP_LABEL}.aetmg"
    listboard_template = "tmg_summary_listboard_template"
    listboard_url = "tmg_summary_listboard_url"
    listboard_panel_style = "warning"
    listboard_model = "edc_action_item.actionitem"
    listboard_panel_title = "TMG: Events Summary"
    listboard_view_permission_codename = "edc_dashboard.view_tmg_listboard"

    model_wrapper_cls = TmgActionItemModelWrapper
    navbar_selected_item = "tmg_home"
    ordering = "-report_datetime"
    paginate_by = 25
    search_form_url = "tmg_summary_listboard_url"
    action_type_names = [
        AE_TMG_ACTION,
        DEATH_REPORT_TMG_ACTION,
        DEATH_REPORT_ACTION,
        AE_FOLLOWUP_ACTION,
    ]
    search_fields = [
        "subject_identifier",
        "action_identifier",
        "parent_action_item__action_identifier",
        "related_action_item__action_identifier",
        "user_created",
        "user_modified",
    ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["AE_TMG_ACTION"] = AE_TMG_ACTION
        context["utc_date"] = arrow.now().date()
        return context

    def get_queryset_filter_options(self, request, *args, **kwargs):
        options = super().get_queryset_filter_options(request, *args, **kwargs)
        options.update({"action_type__name__in": self.action_type_names})
        return options

    def update_wrapped_instance(self, model_wrapper):
        model_wrapper.has_reference_obj_permissions = True
        model_wrapper.has_parent_reference_obj_permissions = True
        model_wrapper.has_related_reference_obj_permissions = True
        try:
            self.request.user.groups.get(name=TMG)
        except ObjectDoesNotExist:
            pass
        else:
            if (
                model_wrapper.reference_obj
                and model_wrapper.reference_obj._meta.label_lower == self.ae_tmg_model
            ):
                model_wrapper.has_reference_obj_permissions = (
                    model_wrapper.reference_obj.user_created == self.request.user.username
                )
            if (
                model_wrapper.parent_reference_obj
                and model_wrapper.parent_reference_obj._meta.label_lower == self.ae_tmg_model
            ):  # noqa
                model_wrapper.has_parent_reference_obj_permissions = (
                    model_wrapper.parent_reference_obj.user_created
                    == self.request.user.username
                )  # noqa
            if (
                model_wrapper.related_reference_obj
                and model_wrapper.related_reference_obj._meta.label_lower == self.ae_tmg_model
            ):  # noqa
                model_wrapper.has_related_reference_obj_permissions = (
                    model_wrapper.related_reference_obj.user_created
                    == self.request.user.username
                )  # noqa
        return model_wrapper



from .ae_initial import AeInitialFormValidator  # noqa
from .ae_susar import AeSusarFormValidator  # noqa
from .ae_tmg import AeTmgFormValidator  # noqa
from .death_report import DeathReportFormValidator  # noqa
from .death_report_tmg import DeathReportTmgFormValidator  # noqa



# flake8: noqa
from .ae_followup import AeFollowupModelAdminMixin
from .ae_initial import (
    AeInitialModelAdminMixin,
    default_radio_fields,
    fieldset_part_four,
    fieldset_part_one,
    fieldset_part_three,
)
from .ae_susar import AeSusarModelAdminMixin
from .ae_tmg import AeTmgModelAdminMixin
from .death_report import DeathReportModelAdminMixin
from .death_report_tmg import DeathReportTmgModelAdminMixin
from .modeladmin_mixins import AdverseEventModelAdminMixin, NonAeInitialModelAdminMixin



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin


class DefaultAeSusarFormValidator(FormValidator):
    pass


class AeSusarModelFormMixin(
    FormValidatorMixin, ModelFormSubjectIdentifierMixin, ActionItemFormMixin
):

    form_validator_cls = DefaultAeSusarFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
        help_text="(read-only)",
    )



from django.conf import settings

AE = "AE"
AE_REVIEW = "AE_REVIEW"
AE_ROLE = "ae_role"
AE_SUPER = "AE_SUPER"
ae_codenames = [
    "edc_adverse_event.view_aeclassification",
    "edc_adverse_event.view_causeofdeath",
    "edc_adverse_event.view_saereason",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_aefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_aeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_aesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_deathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_aefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_aeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_aesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_deathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_aefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_aeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_aesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_deathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_deathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_deathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_deathreporttmgsecond",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicaldeathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicaldeathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicaldeathreporttmgsecond",
]
ae_dashboard_tuples = (("edc_dashboard.view_ae_listboard", "Can view AE listboard"),)
ae_navbar_tuples = (("edc_navbar.nav_ae_section", "Can view AE navbar"),)



import arrow
from dateutil import tz
from django import forms
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import DEAD
from edc_utils import convert_php_dateformat

from edc_adverse_event.get_ae_model import get_ae_model


class RequiresDeathReportFormValidatorMixin:

    """A form validator mixin used by forms that refer to the death report.

    For example: off study report, study termination, etc.

        class StudyTerminationFormValidator(
            DeathReportFormValidatorMixin, FormValidator):

            def clean(self):

                self.validate_death_report_if_deceased()
                ...
    """

    offschedule_reason_field = "termination_reason"
    death_date_field = "death_date"  # on this form, e.g offschedule
    death_report_death_date_field = "death_datetime"  # on death report

    def validate_death_report_if_deceased(self):
        """Validates death report exists of termination_reason
        is "DEAD.

        Death "date" is the naive date of the settings.TIME_ZONE datetime.

        Note: uses __date field lookup. If using mysql don't forget
        to load timezone info.
        """

        if self.cleaned_data.get(self.offschedule_reason_field):
            if (
                self.cleaned_data.get(self.offschedule_reason_field).name == DEAD
                and not self.death_report
            ):
                raise forms.ValidationError(
                    {
                        self.offschedule_reason_field: "Patient is deceased, please complete "
                        "death report form first."
                    }
                )
            elif (
                self.cleaned_data.get(self.offschedule_reason_field).name != DEAD
                and self.death_report
            ):
                raise forms.ValidationError(
                    {
                        self.offschedule_reason_field: (
                            "Invalid selection. A death report was submitted"
                        )
                    }
                )

        if not self.cleaned_data.get(self.death_date_field) and self.death_report:
            raise forms.ValidationError(
                {
                    self.death_date_field: (
                        "This field is required. A death report was submitted."
                    )
                }
            )
        elif self.cleaned_data.get(self.death_date_field) and self.death_report:
            self.match_date_of_death_or_raise()

    @property
    def subject_identifier(self):
        return self.cleaned_data.get("subject_identifier") or self.instance.subject_identifier

    @property
    def death_report(self):
        """Returns a model instance or None"""
        try:
            return get_ae_model("deathreport").objects.get(
                subject_identifier=self.subject_identifier
            )
        except ObjectDoesNotExist:
            return None

    @property
    def death_report_date(self):
        """Returns the localized death date from the death report"""
        try:
            death_report_date = arrow.get(
                getattr(self.death_report, self.death_report_death_date_field),
                tz.gettz(settings.TIME_ZONE),
            ).date()
        except AttributeError:
            death_report_date = getattr(self.death_report, self.death_report_death_date_field)
        except ValueError:
            death_report_date = None
        return death_report_date

    def match_date_of_death_or_raise(self):
        """Raises an exception if the death date reported here does not match
        that from the Death Report."""
        try:
            death_date = self.cleaned_data.get(self.death_date_field).date()
        except AttributeError:
            death_date = self.cleaned_data.get(self.death_date_field)
        if self.death_report_date != death_date:
            expected = self.death_report_date.strftime(
                convert_php_dateformat(settings.SHORT_DATE_FORMAT)
            )
            got = death_date.strftime(convert_php_dateformat(settings.SHORT_DATE_FORMAT))
            raise forms.ValidationError(
                {
                    self.death_date_field: "Date does not match Death Report. "
                    f"Expected {expected}. Got {got}."
                }
            )
        return None



from django.contrib import admin
from django.template.loader import render_to_string
from edc_action_item import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ModelAdminActionItemMixin
from edc_model_admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..forms import AeSusarForm
from ..templatetags.edc_adverse_event_extras import (
    format_ae_susar_description,
    select_description_template,
)
from .modeladmin_mixins import AdverseEventModelAdminMixin, NonAeInitialModelAdminMixin


class AeSusarModelAdminMixin(
    ModelAdminSubjectDashboardMixin,
    NonAeInitialModelAdminMixin,
    AdverseEventModelAdminMixin,
    ModelAdminActionItemMixin,
):

    form = AeSusarForm

    list_display = [
        "subject_identifier",
        "dashboard",
        "description",
        "initial_ae",
        "user",
    ]

    list_filter = ("report_datetime", "submitted_datetime")

    search_fields = [
        "subject_identifier",
        "action_identifier",
        "ae_initial__action_identifier",
        "ae_initial__tracking_identifier",
        "tracking_identifier",
    ]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "ae_initial",
                    "report_datetime",
                    "submitted_datetime",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {"report_status": admin.VERTICAL}

    def description(self, obj):
        """Returns a formatted comprehensive description."""
        context = format_ae_susar_description({}, obj, 50)
        return render_to_string(select_description_template("aesusar"), context)



from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.db.models.signals import m2m_changed, post_delete, post_save
from django.dispatch.dispatcher import receiver
from edc_constants.constants import NO, YES
from edc_notification.models import Notification
from edc_utils import get_utcnow

from edc_adverse_event.constants import DEATH_REPORT_TMG_ACTION

from .auth_objects import TMG
from .constants import AE_TMG_ACTION
from .get_ae_model import get_ae_model

AeInitial = get_ae_model("AeInitial")
AeSusar = get_ae_model("AeSusar")


@receiver(m2m_changed, weak=False, dispatch_uid="update_ae_notifications_for_tmg_group")
def update_ae_notifications_for_tmg_group(
    action, instance, reverse, model, pk_set, using, **kwargs
):

    try:
        instance.userprofile
    except AttributeError:
        pass
    else:
        try:
            tmg_ae_notification = Notification.objects.get(name=AE_TMG_ACTION)
        except ObjectDoesNotExist:
            pass
        else:
            try:
                with transaction.atomic():
                    instance.groups.get(name=TMG)
            except ObjectDoesNotExist:
                instance.userprofile.email_notifications.remove(tmg_ae_notification)
            else:
                instance.userprofile.email_notifications.add(tmg_ae_notification)


@receiver(post_save, sender=AeSusar, weak=False, dispatch_uid="update_ae_initial_for_susar")
def update_ae_initial_for_susar(sender, instance, raw, update_fields, **kwargs):
    if not raw and not update_fields:
        if instance.submitted_datetime:
            if instance.ae_initial.susar_reported != YES:
                instance.ae_initial.susar = YES
                instance.ae_initial.susar_reported = YES
                instance.ae_initial.save(update_fields=["susar", "susar_reported"])
        elif instance.ae_initial.susar_reported != NO:
            instance.ae_initial.susar = YES
            instance.ae_initial.susar_reported = NO
            instance.ae_initial.save(update_fields=["susar", "susar_reported"])


@receiver(
    post_save,
    sender=AeInitial,
    weak=False,
    dispatch_uid="update_ae_initial_susar_reported",
)
def update_ae_initial_susar_reported(sender, instance, raw, update_fields, **kwargs):
    if not raw and not update_fields:
        if instance.susar == YES and instance.susar_reported == YES:
            try:
                with transaction.atomic():
                    AeSusar.objects.get(ae_initial=instance)
            except ObjectDoesNotExist:
                AeSusar.objects.create(ae_initial=instance, submitted_datetime=get_utcnow())


@receiver(post_delete, sender=AeSusar, weak=False, dispatch_uid="post_delete_ae_susar")
def post_delete_ae_susar(instance, **kwargs):
    if instance.ae_initial.susar == YES and instance.ae_initial.susar_reported != NO:
        instance.ae_initial.susar_reported = NO
        instance.ae_initial.save()


@receiver(m2m_changed, weak=False, dispatch_uid="update_death_notifications_for_tmg_group")
def update_death_notifications_for_tmg_group(
    action, instance, reverse, model, pk_set, using, **kwargs
):

    try:
        instance.userprofile
    except AttributeError:
        pass
    else:
        try:
            tmg_death_notification = Notification.objects.get(name=DEATH_REPORT_TMG_ACTION)
        except ObjectDoesNotExist:
            pass
        else:
            try:
                instance.groups.get(name=TMG)
            except ObjectDoesNotExist:
                instance.userprofile.email_notifications.remove(tmg_death_notification)
            else:
                instance.userprofile.email_notifications.add(tmg_death_notification)



import os
from textwrap import wrap

import arrow
from django import template
from django.conf import settings
from django.template.loader import select_template
from django.utils.safestring import mark_safe
from edc_constants.constants import OTHER, YES

register = template.Library()


def select_ae_template(relative_path):
    """Returns a template object."""
    local_path = f"{settings.ADVERSE_EVENT_APP_LABEL}/bootstrap{settings.EDC_BOOTSTRAP}/"
    default_path = f"edc_adverse_event/bootstrap{settings.EDC_BOOTSTRAP}/"
    return select_template(
        [
            os.path.join(local_path, relative_path),
            os.path.join(default_path, relative_path),
        ]
    )


def select_description_template(model):
    """Returns a template name."""
    return select_ae_template(f"{model}_description.html").template.name


@register.inclusion_tag(
    f"edc_adverse_event/bootstrap{settings.EDC_BOOTSTRAP}/"
    f"tmg/tmg_ae_listboard_result.html",
    takes_context=True,
)
def tmg_listboard_results(context, results, empty_message=None):
    context["results"] = results
    context["empty_message"] = empty_message
    return context


@register.inclusion_tag(select_description_template("aeinitial"), takes_context=True)
def format_ae_description(context, ae_initial, wrap_length):
    context["utc_date"] = arrow.now().date()
    context["SHORT_DATE_FORMAT"] = settings.SHORT_DATE_FORMAT
    context["OTHER"] = OTHER
    context["YES"] = YES
    context["ae_initial"] = ae_initial
    context["sae_reason"] = mark_safe(
        "<BR>".join(wrap(ae_initial.sae_reason.name, wrap_length or 35))
    )
    context["ae_description"] = mark_safe(
        "<BR>".join(wrap(ae_initial.ae_description, wrap_length or 35))
    )
    return context


@register.inclusion_tag(select_description_template("aefollowup"), takes_context=True)
def format_ae_followup_description(context, ae_followup, wrap_length):
    context["utc_date"] = arrow.now().date()
    context["SHORT_DATE_FORMAT"] = settings.SHORT_DATE_FORMAT
    context["OTHER"] = OTHER
    context["YES"] = YES
    context["ae_followup"] = ae_followup
    context["ae_initial"] = ae_followup.ae_initial
    context["sae_reason"] = mark_safe(
        "<BR>".join(wrap(ae_followup.ae_initial.sae_reason.name, wrap_length or 35))
    )
    context["relevant_history"] = mark_safe(
        "<BR>".join(wrap(ae_followup.relevant_history, wrap_length or 35))
    )
    context["ae_description"] = mark_safe(
        "<BR>".join(wrap(ae_followup.ae_initial.ae_description, wrap_length or 35))
    )
    return context


@register.inclusion_tag(select_description_template("aesusar"), takes_context=True)
def format_ae_susar_description(context, ae_susar, wrap_length):
    context["utc_date"] = arrow.now().date()
    context["SHORT_DATE_FORMAT"] = settings.SHORT_DATE_FORMAT
    context["OTHER"] = OTHER
    context["YES"] = YES
    context["ae_susar"] = ae_susar
    context["ae_initial"] = ae_susar.ae_initial
    context["sae_reason"] = mark_safe(
        "<BR>".join(wrap(ae_susar.ae_initial.sae_reason.name, wrap_length or 35))
    )
    context["ae_description"] = mark_safe(
        "<BR>".join(wrap(ae_susar.ae_initial.ae_description, wrap_length or 35))
    )
    return context



from django.conf import settings

TMG = "TMG"
TMG_REVIEW = "TMG_REVIEW"
TMG_ROLE = "tmg"

tmg_codenames = [
    "edc_appointment.view_appointment",
    "edc_appointment.view_historicalappointment",
    "edc_dashboard.view_screening_listboard",
    "edc_dashboard.view_subject_listboard",
    "edc_dashboard.view_subject_review_listboard",
    "edc_dashboard.view_tmg_listboard",
    "edc_navbar.nav_screening_section",
    "edc_navbar.nav_subject_section",
    "edc_navbar.nav_tmg_section",
    "edc_adverse_event.view_aeclassification",
    "edc_adverse_event.view_causeofdeath",
    "edc_adverse_event.view_saereason",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_aetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_deathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.add_deathreporttmgsecond",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_aetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_deathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.change_deathreporttmgsecond",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_aetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_deathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.delete_deathreporttmgsecond",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_aetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_deathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_deathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_deathreporttmgsecond",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaefollowup",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaeinitial",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaesusar",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicalaetmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicaldeathreport",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicaldeathreporttmg",
    f"{settings.ADVERSE_EVENT_APP_LABEL}.view_historicaldeathreporttmgsecond",
]
tmg_view_codenames = [c for c in tmg_codenames if "view_" in c]
tmg_dashboard_tuples = (("edc_dashboard.view_tmg_listboard", "Can view TMG Listboard"),)
tmg_navbar_tuples = (("edc_navbar.nav_tmg_section", "Can view TMG navbar"),)



from .ae_initial_ae_model_mixin import AeInitialAeModelMixin  # noqa
from .ae_initial_fields_model_mixin import AeInitialFieldsModelMixin  # noqa
from .ae_initial_methods_model_mixin import AeInitialMethodsModelMixin  # noqa
from .ae_initial_model_mixin import AeInitialModelMixin  # noqa
from .ae_initial_sae_model_mixin import AeInitialSaeModelMixin  # noqa
from .ae_initial_susar_model_mixin import AeInitialSusarModelMixin  # noqa
from .ae_initial_tmg_model_mixin import AeInitialTmgModelMixin  # noqa



import arrow
from dateutil.relativedelta import relativedelta
from edc_consent.consent import Consent
from edc_consent.site_consents import site_consents
from edc_constants.constants import FEMALE, MALE

v1 = Consent(
    "adverse_event_app.subjectconsent",
    version="1",
    start=arrow.utcnow().floor("hour") - relativedelta(years=1),
    end=arrow.utcnow().ceil("hour") + relativedelta(years=1),
    age_min=18,
    age_is_adult=18,
    age_max=110,
    gender=[MALE, FEMALE],
)

site_consents.register(v1)



from copy import copy

from django.contrib import admin
from edc_action_item import action_fields, action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ModelAdminActionItemMixin
from edc_model_admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin


class DeathReportModelAdminMixin(ModelAdminSubjectDashboardMixin, ModelAdminActionItemMixin):

    form = None

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                    "death_datetime",
                    "study_day",
                    "death_as_inpatient",
                )
            },
        ),
        (
            "Opinion of Local Study Doctor",
            {"fields": ("cause_of_death", "cause_of_death_other", "narrative")},
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "death_as_inpatient": admin.VERTICAL,
        "cause_of_death": admin.VERTICAL,
    }

    list_display = (
        "subject_identifier",
        "dashboard",
        "report_datetime",
        "cause_of_death",
        "death_datetime",
        "action_item",
        "parent_action_item",
    )

    list_filter = ("report_datetime", "death_datetime", "cause_of_death")

    search_fields = ["subject_identifier", "action_identifier", "tracking_identifier"]

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        action_flds.remove("action_identifier")
        fields = list(action_flds) + list(fields)
        return fields



from django.urls.conf import path
from edc_protocol.protocol import Protocol

from edc_adverse_event.admin_site import edc_adverse_event_admin

from .views import AeHomeView, ClosedTmgAeListboardView
from .views import DeathListboardView as TmgDeathListboardView
from .views import NewTmgAeListboardView, OpenTmgAeListboardView
from .views import SummaryListboardView as TmgSummaryListboardView
from .views import TmgHomeView

app_name = "edc_adverse_event"


urlpatterns = NewTmgAeListboardView.urls(
    namespace=app_name,
    label="new_tmg_ae_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += OpenTmgAeListboardView.urls(
    namespace=app_name,
    label="open_tmg_ae_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += ClosedTmgAeListboardView.urls(
    namespace=app_name,
    label="closed_tmg_ae_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += TmgDeathListboardView.urls(
    namespace=app_name,
    label="tmg_death_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += TmgSummaryListboardView.urls(
    namespace=app_name,
    label="tmg_summary_listboard",
    identifier_pattern=Protocol().subject_identifier_pattern,
)
urlpatterns += [
    path("tmg/", TmgHomeView.as_view(), name="tmg_home_url"),
    path("ae/", AeHomeView.as_view(), name="ae_home_url"),
    path("admin/", edc_adverse_event_admin.urls),
    path("", AeHomeView.as_view(), name="home_url"),
]



from django.utils.safestring import mark_safe
from edc_action_item.action_with_notification import ActionWithNotification
from edc_constants.constants import CLOSED, HIGH_PRIORITY, NO, YES

from ..constants import (
    ADVERSE_EVENT_ADMIN_SITE,
    ADVERSE_EVENT_APP_LABEL,
    DEATH_REPORT_ACTION,
    DEATH_REPORT_TMG_ACTION,
    DEATH_REPORT_TMG_SECOND_ACTION,
)


class DeathReportTmgAction(ActionWithNotification):
    name = DEATH_REPORT_TMG_ACTION
    display_name = "TMG Death Report (1st) pending"
    notification_display_name = "TMG Death Report (1st)"
    parent_action_names = [DEATH_REPORT_ACTION]
    related_reference_fk_attr = "death_report"
    priority = HIGH_PRIORITY
    create_by_user = False
    color_style = "info"
    show_link_to_changelist = True
    singleton = True
    instructions = mark_safe("This report is to be completed by the TMG only.")

    reference_model = f"{ADVERSE_EVENT_APP_LABEL}.deathreporttmg"
    related_reference_model = f"{ADVERSE_EVENT_APP_LABEL}.deathreport"
    admin_site_name = ADVERSE_EVENT_ADMIN_SITE

    def get_next_actions(self):
        """Returns a DeathReportTmgSecondAction if the
        submitted TMG report does not match the cause of death
        of the original death report.
        """
        next_actions = []
        if self.reference_obj.cause_of_death_agreed == NO:
            next_actions = [DEATH_REPORT_TMG_SECOND_ACTION]
        return next_actions

    def reopen_action_item_on_change(self):
        """Do not reopen if status is CLOSED."""
        return self.reference_obj.report_status != CLOSED

    def close_action_item_on_save(self):
        """Close if report status is CLOSED."""
        if self.reference_obj.cause_of_death_agreed == YES:
            self.delete_children_if_new(parent_action_item=self.action_item)
        return self.reference_obj.report_status == CLOSED



from .ae_susar_fields_model_mixin import AeSusarFieldsModelMixin  # noqa
from .ae_susar_methods_model_mixin import AeSusarMethodsModelMixin  # noqa
from .ae_susar_model_mixin import AeSusarModelMixin  # noqa



from .ae_followup_fields_model_mixin import AeFollowupFieldsModelMixin  # noqa
from .ae_followup_methods_model_mixin import AeFollowupMethodsModelMixin  # noqa
from .ae_followup_model_mixin import AeFollowupModelMixin  # noqa



from django.db import models
from edc_action_item.managers import (
    ActionIdentifierManager,
    ActionIdentifierSiteManager,
)
from edc_action_item.models import ActionModelMixin
from edc_identifier.model_mixins import (
    NonUniqueSubjectIdentifierFieldMixin,
    TrackingModelMixin,
)
from edc_model.models import ReportStatusModelMixin
from edc_sites.models import SiteModelMixin

from edc_adverse_event.constants import AE_SUSAR_ACTION

from .ae_susar_fields_model_mixin import AeSusarFieldsModelMixin
from .ae_susar_methods_model_mixin import AeSusarMethodsModelMixin


class AeSusarModelMixin(
    NonUniqueSubjectIdentifierFieldMixin,
    AeSusarFieldsModelMixin,
    AeSusarMethodsModelMixin,
    ReportStatusModelMixin,
    SiteModelMixin,
    ActionModelMixin,
    TrackingModelMixin,
    models.Model,
):

    action_name = AE_SUSAR_ACTION

    tracking_identifier_prefix = "AS"

    on_site = ActionIdentifierSiteManager()

    objects = ActionIdentifierManager()

    class Meta:
        abstract = True
        verbose_name = "AE SUSAR Report"
        indexes = [
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])
        ]
